Attribute VB_Name = "modExportToExcel"
Option Explicit
' This module takes in a recordset passed to ExportToExcel.
' It uses the constants to determine the location of the initial fields.
' It exports the recordset to excel and formats it.
' It includes titles and the date.
' gTitle and gWksName must be set before the procedure is called.

'This can be done using the following code:
'       modExportToExcel.gExcelTitle = "My Title"
'       modExportToExcel.gWksName = "My Worksheet"
'       Call ExportToExcel(rsHeader)
 

'///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
'////////////////       CONSTANT DECLARATIONS             /////////////////////////////////////////////////////////////
'///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
'Change constants to appropriate values

Private Const cFieldHeaderRow = 4
Private Const cFirstDataRow = 5
'Private Const cFirstDataCol = "A"   used with AddData OLD
Private Const cFirstDataCol = 1
Private Const cFont = "Arial"
Private Const cFieldHeadingColour = 34   'light green colour fill

'///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
'////////////////       VARIABLE DECLARATIONS             /////////////////////////////////////////////////////////////
'///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

Private XL As Excel.Application
Private XLB As Excel.Workbook
Private XLS As Excel.Worksheet
Private XLR As Excel.Range

Public gExcelTitle As String         'Title of excel report - Set before the code is called
Public gWksName As String    'Name on worksheet tab - Set before the code is called.

Private ColCount As Long 'The number of data columns
Private RowCount As Long ' The number of data rows

Private rsExport As ADODB.Recordset

'///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
'////////////////       EXPORT TO EXCEL            ///////////////////////////////////////////////////////////////////////////
'///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

Public Sub ExportToExcel(rs As ADODB.Recordset)
    On Error GoTo ErrHandle
    '^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
    'Set Global variable
    Set rsExport = rs
    
    'Start Excel
    Call StartExcel

    'Copy title, Date, Etc into Excel
    Call AddTitlesToSpreadsheet
    
    'Create Field Headings
    Call AddFieldHeadings
    
    'Copy data from recordset
    Call AddData
    
    'Format the spreadsheet
    Call FormatSpreadsheet
    
    'Reset the excel objects
    Call CloseExcelObjects
 
    '^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
    Exit Sub
ErrHandle:
    MsgBox "Error Number:  " & Err.Number & vbCrLf & Err.Description, _
    vbCritical, "Error in ' modExportToExcel - ExportToExcel ' procedure "
End Sub

'///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
'////////////////       START EXCEL            ///////////////////////////////////////////////////////////////////////////////////
'///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

Private Sub StartExcel()
    On Error GoTo ErrHandle
    '^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
'    Set XL = New Excel.Application
   Set XL = GetObject(, "Excel.application")  ' IF NOT OPEN, WILL PICK UP IN ERROR HANDLING
    XL.Visible = True
    Set XLB = XL.Workbooks.Add
    Set XLS = XLB.Worksheets("Sheet1")
    'synchronize worksheet name and gWksName
    gWksName = Trim(gWksName)
    If gWksName <> "" Then
        'rename the worksheet
        XLS.Name = gWksName
    Else
        'change gwksname from "" to "sheetX"
        gWksName = XLB.Name
    End If
    
    XLB.Activate
    XLS.Activate
 '   ActiveWindow.Zoom = 80
 
    '^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
    Exit Sub
ErrHandle:

    Select Case Err.Number
        Case 429
            Set XL = New Excel.Application
            Resume Next
        Case Else
            MsgBox "Error Number:  " & Err.Number & vbCrLf & Err.Description, _
            vbCritical, "Error in ' modExportToExcel - StartExcel ' procedure "
            Err.Raise Err.Number
        End Select
End Sub

'///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
'////////////////       ADD TITLES TO WORKSHEER          ////////////////////////////////////////////////////////////
'///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

Private Sub AddTitlesToSpreadsheet()
    On Error GoTo ErrHandle
    '^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
    Set XLR = XLS.Range("A1")
    With XLR
        .Value = gExcelTitle
        With .Font
            .Name = cFont
            .Size = 14
            .Bold = True
        End With 'font
    
    End With
    
    Set XLR = XLS.Range("a2")
    With XLR
        .Value = Format(Date, "long date") & " " & Format(Time, "medium time")
        With .Font
            .Name = cFont
            .Size = 11
            .Bold = True
        End With  'Font
    End With    ' range
 
    '^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
    Exit Sub
ErrHandle:
    MsgBox "Error Number:  " & Err.Number & vbCrLf & Err.Description, _
    vbCritical, "Error in ' modExportToExcel - AddTitlesToSpreadsheet ' procedure "
    Err.Raise Err.Number
End Sub

'///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
'////////////////       ADD FIELD HEADINGS            /////////////////////////////////////////////////////////////////////
'///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
Private Sub AddFieldHeadings()
    On Error GoTo ErrHandle
    '^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^

    'At end of procedure, ColCount = number of data columns
    
    Dim fld As ADODB.Field
    
 '   Set XLR = XLS.Range(cFirstDataCol & cFieldHeaderRow)
    Set XLR = XLS.Cells(cFieldHeaderRow, cFirstDataCol)
    
    ColCount = 0
    
    For Each fld In rsExport.Fields
        ColCount = ColCount + 1
        XLR.Value = fld.Name
        With XLR.Font
            .Name = cFont
            .Size = 10
            .Bold = True
        End With  'font
        XLR.Interior.ColorIndex = cFieldHeadingColour
        XLR.Interior.Pattern = xlSolid
        
        Set XLR = XLR.Offset(0, 1)
    Next fld
    
    '^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
    Exit Sub
ErrHandle:
    MsgBox "Error Number:  " & Err.Number & vbCrLf & Err.Description, _
    vbCritical, "Error in ' modExportToExcel - AddFieldHeadings ' procedure "
    Err.Raise Err.Number
End Sub


'///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
'////////////////       ADD DATA            ////////////////////////////////////////////////////////////////////////////////////////
'///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

Private Sub AddData()
    On Error GoTo ErrHandle
    '^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
    Dim fld As ADODB.Field
    Dim CurrRow As Long
    '^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
    Dim RowArray()
    ReDim RowArray(1 To ColCount)
    Dim iCol As Integer
    
    
    CurrRow = cFirstDataRow
    RowCount = 0
    
    rsExport.MoveFirst
    
    Do Until rsExport.EOF
        Set XLR = XLS.Range(XLS.Cells(CurrRow, cFirstDataCol), XLS.Cells(CurrRow, cFirstDataCol + ColCount - 1))
        iCol = 1
        For Each fld In rsExport.Fields
            RowArray(iCol) = fld.Value
            iCol = iCol + 1
        Next fld
        XLR = RowArray
        XLR.Font.Name = cFont
        XLR.Font.Size = 10
        CurrRow = CurrRow + 1
        RowCount = RowCount + 1
        rsExport.MoveNext
    Loop
 
    rsExport.MoveFirst

    '^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
    Exit Sub
ErrHandle:
    MsgBox "Error Number:  " & Err.Number & vbCrLf & Err.Description, _
    vbCritical, "Error in ' modExportToExcel - AddData ' procedure "
    Err.Raise Err.Number
End Sub

'///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
'///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
'////////////////       ADD DATA OLD           ////////////////////////////////////////////////////////////////////////////////////////
'///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

Private Sub AddData_OLD()
    On Error GoTo ErrHandle
    '^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
    Dim fld As ADODB.Field
    Dim CurrRow As Long
    
    CurrRow = cFirstDataRow
    RowCount = 0
    
    rsExport.MoveFirst
    
    Do Until rsExport.EOF
        Set XLR = XLS.Range(cFirstDataCol & CurrRow)
        For Each fld In rsExport.Fields
            XLR.Value = fld.Value
            XLR.Font.Name = cFont
            XLR.Font.Size = 10
            Set XLR = XLR.Offset(0, 1)
        Next fld
        CurrRow = CurrRow + 1
        RowCount = RowCount + 1
        rsExport.MoveNext
    Loop
 
    rsExport.MoveFirst

    '^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
    Exit Sub
ErrHandle:
    MsgBox "Error Number:  " & Err.Number & vbCrLf & Err.Description, _
    vbCritical, "Error in ' modExportToExcel - AddData ' procedure "
    Err.Raise Err.Number
End Sub

'///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
'////////////////       FORMAT SPREADSHEET            /////////////////////////////////////////////////////////////////
'///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

Private Sub FormatSpreadsheet()
    On Error GoTo ErrHandle
    '^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
    'Set XLR = XLS.Range(cFirstDataCol & cFieldHeaderRow)  Used for OLD AddData()
    Set XLR = XLS.Cells(cFieldHeaderRow, cFirstDataCol)

    Set XLR = XLR.Resize(RowCount + 1, ColCount)
    
    With XLR
        .Borders(xlDiagonalDown).LineStyle = xlNone
        .Borders(xlDiagonalUp).LineStyle = xlNone

        With .Borders(xlEdgeLeft)
            .LineStyle = xlContinuous
            .Weight = xlThin
            .ColorIndex = xlAutomatic
        End With 'borders(xlEdgeLeft)
        With .Borders(xlEdgeTop)
            .LineStyle = xlContinuous
            .Weight = xlThin
            .ColorIndex = xlAutomatic
        End With 'Borders(xlEdgeTop)
        With .Borders(xlEdgeBottom)
            .LineStyle = xlContinuous
            .Weight = xlThin
            .ColorIndex = xlAutomatic
            End With 'Borders(xlEdgeBottom)
        With .Borders(xlEdgeRight)
            .LineStyle = xlContinuous
            .Weight = xlThin
            .ColorIndex = xlAutomatic
        End With   'Borders(xlEdgeRight)
        With .Borders(xlInsideVertical)
            .LineStyle = xlContinuous
            .Weight = xlThin
            .ColorIndex = xlAutomatic
        End With  'Borders(xlInsideVertical)
        With .Borders(xlInsideHorizontal)
            .LineStyle = xlContinuous
            .Weight = xlThin
            .ColorIndex = xlAutomatic
        End With  'Borders(xlInsideHorizontal)
    End With  'XLR
        
        'Format cell widths
        With XLS
            .Cells.Select
            .Cells.ColumnWidth = 70
            .Cells.EntireRow.AutoFit
            .Cells.ColumnWidth = 10
            .Cells.EntireColumn.AutoFit
            .Columns("A:A").ColumnWidth = 12
            .Range("A3").Select
            
            XLB.Activate
            XLS.Activate

            DoEvents
        
        End With
    
    Exit Sub
ErrHandle:
    MsgBox "Error Number:  " & Err.Number & vbCrLf & Err.Description, _
    vbCritical, "Error in ' modExportToExcel - FormatSpreadsheet ' procedure "
    Err.Raise Err.Number
End Sub

Private Sub CloseExcelObjects()
    Set XLR = Nothing
    Set XLS = Nothing
    Set XLB = Nothing
    Set XL = Nothing
    Set rsExport = Nothing
End Sub
